"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const cache_1 = __importDefault(require("./cache"));
const complete_1 = __importDefault(require("./complete"));
const createDiagnostics_1 = __importDefault(require("./createDiagnostics"));
const createConnection_1 = __importDefault(require("./createConnection"));
const yargs_1 = require("yargs");
const SettingStore_1 = __importDefault(require("./SettingStore"));
const getDatabaseClient_1 = __importDefault(require("./database_libs/getDatabaseClient"));
const initializeLogging_1 = __importDefault(require("./initializeLogging"));
const log4js = __importStar(require("log4js"));
function createServer() {
    let connection = createConnection_1.default(yargs_1.argv.method || 'node-ipc');
    initializeLogging_1.default();
    const logger = log4js.getLogger();
    let documents = new vscode_languageserver_1.TextDocuments();
    documents.listen(connection);
    let schema = [];
    connection.onInitialize((params) => {
        logger.debug(`onInitialize: ${params.rootPath}`);
        if (params.rootPath) {
            SettingStore_1.default.getInstance().setSettingFromFile(`${params.rootPath}/.sqllsrc.json`);
        }
        SettingStore_1.default.getInstance().on('change', async () => {
            try {
                const client = getDatabaseClient_1.default(SettingStore_1.default.getInstance().getSetting());
                schema = await client.getSchema();
                logger.debug('get schema');
                logger.debug(JSON.stringify(schema));
            }
            catch (e) {
                logger.error(e);
            }
        });
        return {
            capabilities: {
                textDocumentSync: documents.syncKind,
                completionProvider: {
                    resolveProvider: true,
                    triggerCharacters: ['.'],
                }
            }
        };
    });
    connection.onDidChangeTextDocument((params) => {
        logger.debug(`didChangeTextDocument: ${params.textDocument.uri}`);
        cache_1.default.set(params.textDocument.uri, params.contentChanges[0].text);
        const diagnostics = createDiagnostics_1.default(params.textDocument.uri, params.contentChanges[0].text);
        connection.sendDiagnostics(diagnostics);
    });
    connection.onCompletion((docParams) => {
        let text = cache_1.default.get(docParams.textDocument.uri);
        if (!text) {
            cache_1.default.setFromUri(docParams.textDocument.uri);
            text = cache_1.default.get(docParams.textDocument.uri);
        }
        logger.debug(text || '');
        const candidates = complete_1.default(text || '', {
            line: docParams.position.line,
            column: docParams.position.character
        }, schema).candidates;
        logger.debug(candidates.map(v => v.label).join(","));
        return candidates;
    });
    connection.onCompletionResolve((item) => {
        return item;
    });
    connection.listen();
    logger.info('start server');
    return connection;
}
exports.default = createServer;
//# sourceMappingURL=createServer.js.map